
/*A.1. Inference Rules*/
/*A.1.1. Navigating the REA Ontology Primitives and their Super- and Sub-Types*/
is_a(X,X).
is_a(X,Y):-
	rea_lattice(X,is_a,Y).
is_a(X,Z):-
	is_a(X,Y),
	rea_lattice(Y,is_a,Z).
/*A.1.2. Inherited Semantics*/
is_a_role(X,is_a,Z,to,P):-
	rea_role(X,is_a,Y,to,P),
	is_a(Y,Z).
is_a_relator(X,is_a,Z,to,P):-
    rea_relator(X,is_a,Y,to,P),
    is_a(Y,Z).

/*A.2. The Open-edi Business Transaction Ontology*/
/*A.2.1. The OeBTO vocabulary as a subset of REA*/
oebto_vocabulary([product,cash,delivery,payment,trading-partner,stock-flow,transfer-duality,from,to]).
oebto_interface(X):-oebto_vocabulary(L),
	member(X,L).
/*A.2.2. Defining the visibility of OeBTO compliant REA roles and relators*/
oebto_role(X,is_a,Y,to,Z):-
	is_a_role(X,is_a,Y,to,P),
	oebto_interface(Y),
	access(Z,to,P,data).
oebto_relator(X,is_a,Y,to,Z):-
	is_a_relator(X,is_a,Y,to,P),
	oebto_interface(Y),
	access(Z,to,P,data).

/*A.2.3. Mirror (Formal relation)*/
oebto_relator([X,Y],is_a,Z,to,P):-
    access(P,to,Y,data),
    rea_relation(Alpha,mirrors,Z),
    rea_relator([X,_],is_a,Alpha,to,Y).
/*A.2.3.1. Opposing Semantics in Opposing Views*/
rea_relation(from,mirrors,to).
rea_relation(to,mirrors,from).



/*A.3. Deriving a Trading-partner view from the Independent view*/
/*A.3.1. inflow and outflow for transfers*/
dependent_relator([X,Y],is_a,outflow,to,P):-
    oebto_relator([X,Y],is_a,stock-flow,to,P),
    oebto_relator([Y,P],is_a,from,to,P).
dependent_relator([X,Y],is_a,inflow,to,P):-
    oebto_relator([X,Y],is_a,stock-flow,to,P),
    oebto_relator([Y,P],is_a,to,to,P).
/*A.3.2. roles*/
/*A.3.2.1. Resource semantics*/
/*A.3.2.2. Event semantics*/
dependent_role_event(E,is_a,X,to,P):-
    oebto_role(E,is_a,Y,to,P),
    oebto_relator([E,P],is_a,Z,to,P),
    event_pattern(X,is_a,Y,Z).
/*A.3.2.2.1. Event Patterns*/
event_pattern(sale,is_a,delivery,from).
event_pattern(purchase,is_a,delivery,to).
event_pattern(cash-receipt,is_a,payment,to).
event_pattern(cash-disbursement,is_a,payment,from).
/*A.3.2.3. Agent semantics*/
dependent_role_agent(A,is_a,Z,to,P,in,[D,A]):-
    oebto_relator([D,A],is_a,Y,to,P),
    oebto_role(D,is_a,X,to,P),
    agent_pattern(X,Y,Z).
/*A.3.2.3.1. Agent patterns*/
agent_pattern(delivery,to,buyer).
agent_pattern(delivery,from,seller).


/*A.4. REA meta-model (fig. 1)*/
/*A.4.1. RoleMixins and Roles*/
/*A.4.1.1. Economic economic-resources*/
/*A.4.1.2. Economic Events (fig. 1 Duality)*/
/*A.4.1.2.1. View Independent Semantics*/
/*A.4.1.2.2. View Dependent Semantics*/
rea_lattice(increment-event,is_a,economic-event).
    rea_lattice(produce,is_a,increment-event).
    rea_lattice(produce,is_a,transformation).
    rea_lattice(take,is_a,increment-event).
    rea_lattice(take,is_a,transfer).
rea_lattice(decrement-event,is_a,economic-event).
    rea_lattice(consume,is_a,decrement-event).
    rea_lattice(consume,is_a,transformation).
    rea_lattice(give,is_a,decrement-event).
    rea_lattice(give,is_a,transfer).
/*A.4.1.3. Economic Agents (fig. 1 Party)*/
/*A.4.1.3.1. View Independent Semantics*/
rea_lattice(trading-partner,is_a,economic-agent).

/*A.4.2. Relators*/
/*A.4.2.1. Stock-Flow*/
    rea_lattice(inflow,is_a,stock-flow).
    rea_lattice(outflow,is_a,stock-flow).

/*A.4.2.2. Duality*/
    rea_lattice(conversion-duality,is_a,duality).
    rea_lattice(exchange-duality,is_a,duality).

/*A.4.2.3. Participation*/
    rea_lattice(inside-party,is_a,participation).
    rea_lattice(outside-party,is_a,participation).
    rea_lattice(from,is_a,outside-party).
    rea_lattice(to,is_a,outside-party).



/*A.5. REA Value Chain model (fig. 2)*/
/*A.5.1. Extension of the Semantic Lattice*/
/*A.5.1.1. Economic Resources*/
/*A.5.1.1.1. Product*/
rea_lattice(product,is_a,economic-resource).
/*A.5.1.1.2. Cash*/
rea_lattice(cash,is_a,economic-resource).
/*A.5.1.2. Economic Events*/
/*A.5.1.2.1. Delivery*/
rea_lattice(delivery,is_a,transfer).
    rea_lattice(sale,is_a,delivery).
    rea_lattice(sale,is_a,give).
    rea_lattice(purchase,is_a,delivery).
    rea_lattice(purchase,is_a,take).
/*A.5.1.2.2. Payment*/
rea_lattice(payment,is_a,transfer).
    rea_lattice(cash-disbursement,is_a,payment).
    rea_lattice(cash-disbursement,is_a,give).
    rea_lattice(cash-receipt,is_a,payment).
    rea_lattice(cash-receipt,is_a,take).
/*A.5.1.3. Economic Agents*/
rea_lattice(seller,is_a,trading-partner).
rea_lattice(buyer,is_a,trading-partner).

/*A.6. The Open-edi Business Transaction Ontology*/
/*A.6.1. The OeBTO vocabulary as a subset of REA*/
blockchain_vocabulary([product,cash,delivery,payment,trading-partner,stock-flow,transfer-duality,from,to,produce,consume,transformation-duality]).
/*A.6.2. Defining the visibility of OeBTO compliant REA tags*/
blockchain_role(X,is_a,Y):-
	is_a_role(X,is_a,Y,to,_),
	blockchain_interface(Y).
blockchain_relator(X,is_a,Y):-
	is_a_relator(X,is_a,Y,to,_),
	blockchain_interface(Y).
blockchain_interface(X):-blockchain_vocabulary(L),
	member(X,L).
/*A.6.3. Mirror (Formal relation)*/
blockchain_relator(X,is_a,Z):-
    oebto_relator(X,is_a,Z,to,_).
/*A.6.4. Produce & Consume*/
blockchain_relator([X,Y],is_a,inside-party):-
    is_a_role(X,is_a,transformation,to,Y).






